
""" __about__ """

__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "magic-xxl"
__version__ = "0.6.0"
__summary__ = "A collection librairies to work with Flask-Magic"
__uri__ = "https://github.com/mardix/magic-xxl"
__author__ = "Mardix"
__email__ = "mardix@pylot.io"
__license__ = "MIT"
__copyright__ = "(c) 2016 %s" % __author__
